
#include <string>
#include <fstream>
#include <iostream>
#include <unistd.h>

#include <Glob.hh>
#include <support.hh>
#include <os.hh>

#include <inttypes.h>

#include <firmware-match.hh>

class mofstream : public std::ofstream
{
public:
  mofstream (const std::string& file)
    : std::ofstream (file.c_str()) {}
  virtual ~mofstream () {}

  void open (const std::string& file, int mode) {
    std::ofstream::open (file.c_str(), mode);
  }
};

class mifstream : public std::ifstream
{
public:
  mifstream (const std::string& file)
    : std::ifstream (file.c_str()) {}
  virtual ~mifstream () {}
};

void firmware_subsystem::hotplug ()
{
  // parse env vars ...
  /*
    ACTION=%s [add or remove]
    PCI_CLASS=%06X
    PCI_ID=%04X:%04X
    PCI_SLOT_NAME=%s
    PCI_SUBSYS_ID=%04X:%04X
  */

  std::string sysfs_path = "/sys";
  std::string firmware_path = "/lib/firmware";

  // TODO: use /proc for 2.4 kernels ...
  
  const char* dev_path = getenv ("DEVPATH");
  const char* firmware = getenv ("FIRMWARE");
  std::string action = getenv ("ACTION");

  if (action != "add") {
    std::cout << "ACTION != add, nothing to do ..." << std::endl;
    return;
  }

  if (!dev_path) {
    std::cout << "DEVPATH not set" << std::endl;
    exit (1);
  }
  if (!firmware) {
    std::cout << "FIRMWARE not set" << std::endl;
    exit (1);
  }
  
  std::cout << "Loading " << firmware_path + "/" + firmware
	    << " into " << sysfs_path + "/" + dev_path + "/data" << std::endl;
  
  {
    mofstream status_file (sysfs_path + "/" + dev_path + "/loading");
    
    // test for existence ...
    int tries;
    for ( tries = 5; tries > 0 && ! status_file.is_open(); --tries) {
      std::cout << " -- Not yet there - sleeping ..." << std::endl;
      sleep (1);
      status_file.open (sysfs_path + "/" + dev_path + "/loading", 0);
    }
    if (!status_file.is_open()) {
        std::cout << " -- Still not there - giving up ..." << std::endl;
        exit (1);
    }
    std::cout << "Start..." << std::endl;
    status_file << "1";
  }

  // copy firmware file
  mifstream firmware_file (firmware_path + "/" + firmware);
  mofstream data_file (sysfs_path + "/" + dev_path + "/data");

  if (!firmware_file.is_open() || !data_file.is_open() ) {
    if  (!firmware_file.is_open())
      std::cout << "Firmware: " << firmware << "not found." << std::endl;
    else
      std::cout << "data node not there" << std::endl;
    mofstream status_file (sysfs_path + "/" + dev_path + "/loading");
    status_file << "-1";
    exit (1);
  }

  std::cout << "Loading: " << firmware << std::endl;
  
  // TODO: if embeddedSTL streams are more complete:
  // firmware_file >> data_file.rdbuf();

  const int bufsize = 1024;
  char* buf = new char[bufsize];
  int i = 0;
  do {
    i = firmware_file.readsome (buf, bufsize);
    std::cout << "got and writing: " << i << std::endl;
    data_file.write(buf, i);
  } while (i == bufsize);
  firmware_file.close();
  data_file.close();

  std::cout << "End..." << std::endl;
  mofstream status_file (sysfs_path + "/" + dev_path + "/loading");
  status_file << "0";
}
