
#include <stdlib.h>

#include <iostream>
#include <fstream>

#include <string>

#include <vector>
#include <algorithm>

#include "os.hh"
#include "Glob.hh"

static std::vector<std::string> blacklisted_modules;
static std::vector<std::string> loaded_modules;

class blacklist_load_on_startup
{
public:
  blacklist_load_on_startup () {
    std::ifstream f ("/etc/conf/blacklist");
    std::string line;
    while (f.is_open() && !f.eof()) {
      std::getline (f, line);
      if (line.empty())
	continue;
      if (line[0] == '#')
	continue;

      // skip leading and trailing whitespace
      std::string::size_type p1, p2;
      p1 = line.find_first_not_of (" \t");
      p2 = line.find_last_not_of (" \t");

      line = line.substr (p1, p2-p1+1);

      //std::cerr << "-> '" << line << "' : " << line.size() << std::endl;
      blacklisted_modules.push_back (line);
    }
  }
} blacklist_load_on_startup;

std::string kernel_release ()
{
  utsname uts;
  uname (&uts);
  return uts.release;
}

void run_hotplug_agents (const std::string& subsystem)
{
  /* compatibility calls ... */
  {
    Utility::Glob hotplug_agents ("/etc/hotplug.d/" + subsystem + "/*.hotplug");
    for (Utility::Glob::iterator it = hotplug_agents.begin();
	 it != hotplug_agents.end(); ++it)
      {
	std::cout << "Executing: " << *it << std::endl;
	system (*it);
      }
  }
  {
    Utility::Glob hotplug_agents ("/etc/hotplug.d/default/*.hotplug");
    for (Utility::Glob::iterator it = hotplug_agents.begin();
	 it != hotplug_agents.end(); ++it)
      {
	std::string s (*it);
	if (s.find("default/default.hotplug") != std::string::npos)
	  continue;
	
	std::cout << "Executing: " << *it << std::endl;
	system (*it);
      }
  }
}

bool modprobe (const std::string& module)
{
  // avoid loading modules multiple times, during synth runs ...
  std::vector<std::string>::iterator it;

  it = std::find (loaded_modules.begin(), loaded_modules.end(), module);
  if (it != loaded_modules.end())
    return true;

  std::cout << "Matched " << module << ": ";

  // blacklisted?
  it = std::find (blacklisted_modules.begin(), blacklisted_modules.end(), module);
  if (it != blacklisted_modules.end()) {
    std::cout << "blacklisted" << std::endl;
    return true;
  }

  // depending on ACTION, maybe rmmod, but I would rather avoid that -ReneR
  std::cout << "loading" << std::endl;

  std::string cmd = "modprobe " + module;
  bool ret = system (cmd.c_str()) == 0;
  if (ret)
    loaded_modules.push_back (module);

  loaded_modules.push_back (module);
  return ret;
}
