
#include <string>
#include <fstream>
#include <iostream>

#include <Glob.hh>
#include <support.hh>
#include <os.hh>

#include <inttypes.h>

#include <scsi-match.hh>

bool scsi_module::operator== (const scsi_module& other) {
  if (type == other.type) return true;
  
  return false;
}

void scsi_module::read_from_sysfs (const std::string& file) {
  type = read_value_from_file<uint32_t> (file + "/type");
}

void scsi_module::read_from_proc (const std::string& file) {

    /* FIXME */
  
}

const char *__scsi_type[] = 
  {
    "disk"      /* 0 */,
    "tape"      /* 1 */,
    "printer"   /* 2 */,
    "processor" /* 3 */,
    "worm"      /* 4 */,
    "cdrom"     /* 5 */,
    "scanner"   /* 6 */,
    "mod"       /* 7 */,
    "changer"   /* 8 */,
    "comm"      /* 9 */,
    ""          /* 10 */,
    ""          /* 11 */,
    ""          /* 12 */,
    ""          /* 13 */,
    "enclosure" /* 14 */,
    NULL
  };

std::ostream& operator<< (std::ostream& os, const scsi_module& m) {
  os << "type: " << m.type;
  if (m.type < 15) os << " (" << __scsi_type[m.type] << ")";
  
  if (!m.module.empty())
    os << ": Module: " << m.module;
  
  return os;
}

void scsi_module::read_from_env () {
  std::string devpath = getenv("DEVPATH");
  if (devpath.length()) {
    devpath = "/sys" + devpath;
    read_from_sysfs(devpath);
  }
}

// ...

std::vector<scsi_module> scsi_subsystem::modules;

void scsi_subsystem::hotplug ()
{
  parse_modules();
  scsi_module m;
  m.read_from_env ();
  match_modules (m, &scsi_subsystem::hotplug);
}

void scsi_subsystem::hotplug (const scsi_module& m)
{
  if (m.module.size()) {
    modprobe (m.module);
  }
}

void scsi_subsystem::hotplug_all ()
{
  parse_modules();
  for_each (&scsi_subsystem::hotplug);
}

void scsi_subsystem::list ()
{
  std::cout << "SCSI types and matched modules:" << std::endl;
  parse_modules();
  
  for_each (&scsi_subsystem::list);
  std::cout << std::endl;
}

void scsi_subsystem::list (const scsi_module& m)
{
  std::cout << m << std::endl;
}

void scsi_subsystem::match_modules (scsi_module& m,
				   void(scsi_subsystem::*op)(const scsi_module&))
{
  std::vector<scsi_module>::iterator module = modules.begin();
  while (module != modules.end()) {
    module = std::find (module, modules.end(), m);
    if (module != modules.end()) {
      m.module = module->module;
        (this->*op)(m);
	++module;
    }
  }
}

void scsi_subsystem::for_each (void(scsi_subsystem::*op)(const scsi_module&))
{
  /* FIXME/CHECK: DEVPATH=devices/scsi0/0:0:0:0 */
  Utility::Glob scsi_devices ("/sys/bus/scsi/devices/*/?:?:?:?");
  for (Utility::Glob::iterator it = scsi_devices.begin();
       it != scsi_devices.end(); ++it)
    {
      scsi_module m;
      m.read_from_sysfs (*it);
      match_modules (m, op);
    }
}

void scsi_subsystem::parse_modules ()
{
  if (modules.size())
    return;

  scsi_module m;
  
  m.type = 0; m.module = "sd_mod"; modules.push_back (m);
  m.type = 7; m.module = "sd_mod"; modules.push_back (m);
  m.type = 1; m.module = "st"; modules.push_back (m);
  m.type = 4; m.module = "sr_mod"; modules.push_back (m);
  m.type = 5; m.module = "sr_mod"; modules.push_back (m);
  m.type = 5; m.module = "sg" ; modules.push_back (m);
  m.type = 6; m.module = "sg" ; modules.push_back (m);
  m.type = 8; m.module = "sg" ; modules.push_back (m);
}
